#include <gear/memory.h>
#include <log.c/log.h>

void* __g_mem_alloc_named(const char* file, i32 line, const char* resource, sz size) {
    void* ptr = MEM_ALLOC_FUNC(size);
    if(ptr != NULL) {
        if(resource != NULL) {
            log_log(
                LOG_INFO, file, line, "%s allocated successfully (%zu bytes)", resource, size);
            log_log(LOG_DEBUG, file, line, "%s pointer: %p", resource, ptr);
        } else {
            log_log(LOG_INFO, file, line, "%zu bytes allocated successfully", size);
            log_log(LOG_DEBUG, file, line, "Pointer: %p", ptr);
        }

        return ptr;
    } else {
        if(resource != NULL) {
            log_log(
                LOG_FATAL, file, line, "Failed to allocate %s (%zu bytes)", resource, size);
        } else {
            log_log(LOG_FATAL, file, line, "Failed to allocate %zu bytes", size);
        }

        exit(EXIT_FAILURE);
    }
}

void* __g_mem_calloc_named(const char* file, i32 line, const char* resource, sz size) {
    void* ptr = __g_mem_alloc_named(file, line, resource, size);
    memset(ptr, 0, size);
    if(resource != NULL) {
        log_log(LOG_INFO, file, line, "Zeroed %zu bytes for %s", size, resource);
    } else {
        log_log(LOG_INFO, file, line, "Zeroid %zu bytes", size);
    }

    return ptr;
}

void* __g_mem_alloc(const char* file, i32 line, sz size) {
    return __g_mem_alloc_named(file, line, NULL, size);
}

void* __g_mem_calloc(const char* file, i32 line, sz size) {
    return __g_mem_calloc_named(file, line, NULL, size);
}
